@@ -38,15 +38,25 @@
 	 (idr (char-set-union idf char-set:digit)))
     (make-ident-reader idf idr)))
 
+;; @deffn {Procecure} read-js-string ch => ($string "abc") | #f
 (define (read-js-string delim)
   (if (not (or (eq? delim #\') (eq? delim #\"))) #f
       (let iter ((cl '()) (ch (read-char)))
 	(cond ((eq? ch #\\)
 	       (let ((c1 (read-char)))
-		 (if (eq? c1 #\newline)
-		     (iter cl (read-char))
-		     (iter (cons* c1 cl) (read-char)))))
-	      ((eq? ch delim) (list->string (reverse cl)))
+		 (iter
+		  (case c1 		; see sec 7.8.4 in es5.1 spec
+		    ((#\newline) (throw 'js-err "newline in string literal"))
+		    ((#\' #\" #\\) (cons c1 cl))
+		    ((#\b) (cons #\backspace cl))
+		    ((#\f) (cons #\page cl))
+		    ((#\n) (cons #\newline cl))
+		    ((#\r) (cons #\return cl))
+		    ((#\t) (cons #\tab cl))
+		    ((#\v) (cons #\vtab cl))
+		    (else (cons c1 cl)))
+		  (read-char))))
+	      ((eq? ch delim) (cons '$string (list->string (reverse cl))))
 	      (else (iter (cons ch cl) (read-char)))))))
 
 (define gen-js-lexer
@@ -78,7 +88,7 @@
 		  (cons semicolon ";")
 		  (iter (read-char))))
 	     ((and (eqv? ch #\newline) (set! bol #t) #f))
-	     ((read-js-string ch) assc-$)
+	     ((read-js-string ch) => assc-$)
 	     ((read-c-comm ch bol) (iter (read-char)))
 	     ((read-js-ident ch) =>
 	      (lambda (s)
